@@ -1,7 +1,7 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright (c) 2012-2014 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2012-2015 Oracle and/or its affiliates. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common Development
@@ -37,6 +37,7 @@
  * only if the new code is made subject to such option by the copyright
  * holder.
  */
+
 package org.glassfish.jersey.servlet.init;
 
 import java.util.Collections;
@@ -119,7 +120,7 @@ import jersey.repackaged.com.google.common.collect.Maps;
  * {@link ServletContainerInitializer} implementation used for Servlet 3.x deployment.
  *
  * @author Paul Sandoz
- * @author Martin Matula (martin.matula at oracle.com)
+ * @author Martin Matula
  * @author Libor Kramolis (libor.kramolis at oracle.com)
  */
 @HandlesTypes({ Path.class, Provider.class, Application.class, ApplicationPath.class })
@@ -247,7 +248,7 @@ public final class JerseyServletContainerInitializer implements ServletContainer
 
             if (registration.getClassName() != null) {
                 // class name present - complete servlet registration from container point of view
-                Utils.store(resourceConfig, context);
+                Utils.store(resourceConfig, context, registration.getName());
             } else {
                 // no class name - no complete servlet registration from container point of view
                 final ServletContainer servlet = new ServletContainer(resourceConfig);
@@ -297,7 +298,7 @@ public final class JerseyServletContainerInitializer implements ServletContainer
      * Enhance existing servlet configuration.
      */
     private static void addServletWithExistingRegistration(final ServletContext context,
-                                                           final ServletRegistration registration,
+                                                           ServletRegistration registration,
                                                            final Class<? extends Application> clazz,
                                                            final Set<Class<?>> classes) throws ServletException {
         // create a new servlet container for a given app.
@@ -307,33 +308,33 @@ public final class JerseyServletContainerInitializer implements ServletContainer
 
         if (registration.getClassName() != null) {
             // class name present - complete servlet registration from container point of view
-            Utils.store(resourceConfig, context);
+            Utils.store(resourceConfig, context, registration.getName());
         } else {
             // no class name - no complete servlet registration from container point of view
             final ServletContainer servlet = new ServletContainer(resourceConfig);
             final ServletRegistration.Dynamic dynamicRegistration = context.addServlet(clazz.getName(), servlet);
             dynamicRegistration.setAsyncSupported(true);
             dynamicRegistration.setLoadOnStartup(1);
+            registration = dynamicRegistration;
+        }
+        if (registration.getMappings().isEmpty()) {
+            final ApplicationPath ap = clazz.getAnnotation(ApplicationPath.class);
+            if (ap != null) {
+                final String mapping = createMappingPath(ap);
+                if (!mappingExists(context, mapping)) {
+                    registration.addMapping(mapping);
 
-            if (dynamicRegistration.getMappings().isEmpty()) {
-                final ApplicationPath ap = clazz.getAnnotation(ApplicationPath.class);
-                if (ap != null) {
-                    final String mapping = createMappingPath(ap);
-                    if (!mappingExists(context, mapping)) {
-                        dynamicRegistration.addMapping(mapping);
-
-                        LOGGER.log(Level.CONFIG, LocalizationMessages.JERSEY_APP_REGISTERED_MAPPING(clazz.getName(), mapping));
-                    } else {
-                        LOGGER.log(Level.WARNING, LocalizationMessages.JERSEY_APP_MAPPING_CONFLICT(clazz.getName(), mapping));
-                    }
+                    LOGGER.log(Level.CONFIG, LocalizationMessages.JERSEY_APP_REGISTERED_MAPPING(clazz.getName(), mapping));
                 } else {
-                    // Error
-                    LOGGER.log(Level.WARNING, LocalizationMessages.JERSEY_APP_NO_MAPPING_OR_ANNOTATION(clazz.getName(),
-                            ApplicationPath.class.getSimpleName()));
+                    LOGGER.log(Level.WARNING, LocalizationMessages.JERSEY_APP_MAPPING_CONFLICT(clazz.getName(), mapping));
                 }
             } else {
-                LOGGER.log(Level.CONFIG, LocalizationMessages.JERSEY_APP_REGISTERED_APPLICATION(clazz.getName()));
+                // Error
+                LOGGER.log(Level.WARNING, LocalizationMessages.JERSEY_APP_NO_MAPPING_OR_ANNOTATION(clazz.getName(),
+                        ApplicationPath.class.getSimpleName()));
             }
+        } else {
+            LOGGER.log(Level.CONFIG, LocalizationMessages.JERSEY_APP_REGISTERED_APPLICATION(clazz.getName()));
         }
     }
 
